#
# See the file "LICENSE" for the full license governing this code.
#
#
# This function checks if the specfied patches are installed.
# The patch list is an array and has the syntax
#
#    ['/oracle/home/dir:patchid']
#
# It will check all registered Oracle homes for the installed patches.
#
Puppet::Functions.create_function(:'ora_install::ora_patches_installed') do
  dispatch :check_patch_list do
    param 'Array', :patch_list
    return_type 'Boolean'
  end

  def check_patch_list(patch_list)
    Puppet.debug("patch list: #{patch_list}")
    scope = closure_scope
    oracle_homes = scope['ora_install_homes'] || []
    return false if oracle_homes.empty?

    oracle_homes['opatch_version'].each do |key, value|
      if value == 'Invalid'
        fail "Either ORACLE_HOME '#{key}' or '#{key}/OPatch/opatch' not found. Faulty oraInventory entry? Please fix!"
      end
    end
    Puppet.debug("existing homes: #{oracle_homes}")

    patch_homes = Facter.value(:kernel) == 'windows' ? patch_list.map { |p| "#{p.split(':')[0]}:#{p.split(':')[1]}" }.uniq : patch_list.map { |p| p.split(':')[0] }.uniq
    Puppet.debug("homes in patch_list: #{patch_homes}")
    installed_patches = oracle_homes['installed_patches'].nil? ? [] : oracle_homes['installed_patches'].collect { |key, val| val.map { |patch| "#{key}:#{patch}" } }.flatten
    Puppet.debug("installed patches: #{installed_patches}")
    patch_list.all? do |patch|
      if patch.split(':')[2] == 'remove'
        installed_patches.include?("#{patch.split(':')[0]}:#{patch.split(':')[1]}") ? false : true
      else
        installed_patches.include?("#{patch.split(':')[0]}:#{patch.split(':')[1]}")
      end
    end
  end
end
