#
# See the file "LICENSE" for the full license governing this code.
#
#
# This function parses a Hash of patches and looks for resources with
# sub_patches. If they are there, this function will translate it to the
# actual list of patches that need to be on the system.
#
Puppet::Functions.create_function(:'ora_install::ora_physical_patches') do
  dispatch :physical_patches do
    param 'Hash', :patch_list
    return_type 'Array'
  end

  def physical_patches(patch_list)
    return [] if patch_list.nil?
    patch_list.keys.collect do |full_patch_id|
      if patch_list[full_patch_id].key?('ensure')
        if patch_list[full_patch_id]['ensure'].to_s == 'absent'
          remove = true
        end
      end
      if Facter.value(:kernel) == 'windows'
        oh1, oh2, _patch_id = full_patch_id.split(':')
        oracle_home = "#{oh1}:#{oh2}"
      else
        oracle_home, _patch_id = full_patch_id.split(':')
      end
      sub_patches = patch_list[full_patch_id]['sub_patches']
      if sub_patches
        if remove
          sub_patches.collect { |p| "#{oracle_home}:#{p}:remove" }
        else
          sub_patches.collect { |p| "#{oracle_home}:#{p}" }
        end
      else
        if remove
          "#{full_patch_id}:remove"
        else
          full_patch_id
        end
      end
    end.compact.flatten
  end
end
