#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'

Puppet::Type.type(:db_control).provide(:base) do

  confine :true => false # This is NEVER a valid provider. It is just used as a base class

  def self.instances
    fail 'resource list not supported for db_control type'
  end

  def stop_deamons
    # deamon not yet supported on windows
    unless Facter.value(:kernel) == 'windows'
      name        = resource[:instance_name]
      oracle_home = resource[:oracle_product_home_dir]
      output = Puppet::Util::Execution.execute("fuser #{oracle_home}/bin/sqlplus", { :failonfail => false, :combine => true })
      return if output.empty?  # No daemons active
      pids = output.split(':')[1].split().collect {|e| e[0...-1]}
      pids.each do | pid |
        sid = Puppet::Util::Execution.execute("cat /proc/#{pid}/environ | tr '\\0' '\\n' | grep ORACLE_SID=#{name}", { :failonfail => false, :combine => true })
        if sid != ''
          Puppet.debug "Forcing stop on sql daemon with pid #{pid}"
          Puppet::Util::Execution.execute("kill -9 #{pid}", { :failonfail => false, :combine => true })
        end
      end
    end
  end

  def start
    instance_control(:start)
  end

  def stop
    instance_control(:stop, 'immediate')
  end

  def restart
    instance_control :stop if status == :start
    instance_control :start
  end

end
