#
# See the file "LICENSE" for the full license governing this code.
#
require "#{File.dirname(__FILE__)}/base"
require_relative '../../../../../ora_config/lib/puppet_x/enterprisemodules/oracle/settings' if Facter.value(:kernel) == 'windows'
require 'easy_type'

Puppet::Type.type(:db_control).provide(:sqlplus, :parent => :base) do
  #
  # This is bit of a hack. id is always root, but we need to declare a default provider
  #
  defaultfor :id => 'root'
  defaultfor :operatingsystem => :windows

  if Facter.value(:kernel) == 'windows'
    include PuppetX::EnterpriseModules::Oracle::Settings
    include ::EasyType::Encryption
    include ::EasyType::Template
  end

  def instance_control(action, options = nil)
    Puppet.debug "instance action: #{action}"

    command, @succes_output = case action
                              when :start
                                ["startup #{options}", /(?:ORACLE instance started|Running *OracleService)/]
                              when :stop
                                ["shutdown #{options}", /(?:ORACLE instance shut down|Stopped *OracleService)/]
                              else
                                fail "internal error unknown action #{action}"
                              end

    Puppet.info "instance action: #{action} with command #{command}"
    @output = sql command
    fail(@output) if unsuccessful?

    Puppet.info "instance result: #{@output}"
    stop_deamons if action == :stop
  end

  def status
    name = resource[:instance_name]

    kernel = Facter.value(:kernel)

    ps_bin = (kernel != 'SunOS' || (kernel == 'SunOS' && Facter.value(:kernelrelease) == '5.11')) ? '/bin/ps' : '/usr/ucb/ps'
    ps_arg = kernel == 'SunOS' ? 'awwx' : '-ef'

    command = Puppet::Util::Platform.windows? ? 'tasklist /svc' : "#{ps_bin} #{ps_arg} | /bin/grep -v grep | /bin/grep 'ora_smon_#{name}'"

    Puppet.debug "instance_status #{command}"
    output = Puppet::Util::Execution.execute(command, { :failonfail => false, :combine => true })
    if Puppet::Util::Platform.windows?
      output.scan(/oracle.exe.*OracleService#{name}/).empty? ? :stop : :start
    else
      output.scan(/ora_smon_#{name}/).empty? ? :stop : :start
    end
  end

  private

  def unsuccessful?
    @output.scan(@succes_output).empty?
  end

  def sql(command)
    name        = resource[:instance_name]
    oracle_home = resource[:oracle_product_home_dir]
    user        = resource[:os_user]
    if Facter.value(:kernel) == 'windows'
      svc_state = command.scan(/shutdown/).empty? ? 'Running' : 'Stopped'
      Puppet::Util::Execution.execute(%[powershell "Set-Service -Name OracleService#{name} -Status #{svc_state} -PassThru"])
    else
      command = "NLS_LANG=english sqlplus /nolog <<-EOF
      connect / as sysdba
      #{command}
      EOF"
          `su - #{user} -c 'export ORACLE_HOME="#{oracle_home}";export PATH="#{oracle_home}/bin:$PATH";export ORACLE_SID="#{name}";unset TNS_ADMIN;unset TWO_TASK;export LD_LIBRARY_PATH="#{oracle_home}/lib";#{command}'`
    end
  end
end
