#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
if Facter.value(:kernel) == 'windows'
  require 'win32/registry'
  require 'puppet_x/enterprisemodules/ora_install/get_homes'
end

Puppet::Type.type(:db_listener).provide(:db_listener) do
  mk_resource_methods

  def self.instances
    []
  end

  def listener_control(action)
    oracle_base  = resource[:oracle_base_dir]
    listenername = resource[:listener_name]

    if action == :start
      oracle_home     = resource[:oracle_home_dir]
      user            = resource[:os_user]
      listener_action = "start #{listenername}"
    else
      oracle_home     = oracle_home_dir
      user            = os_user
      listener_action = "stop #{listenername}"
    end

    if Facter.value(:kernel) == 'windows'
      command = "#{oracle_home}\\bin\\lsnrctl.exe #{listener_action}"
      output = `#{command}`
    else
      command = "#{oracle_home}/bin/lsnrctl #{listener_action}"
      full_command = "export ORACLE_HOME=#{oracle_home};export ORACLE_BASE=#{oracle_base};export LD_LIBRARY_PATH=#{oracle_home}/lib;cd #{oracle_home};#{command}"
      options = { :failonfail => true, :combine => true, :uid => user }
      Puppet.info "listener action: #{action} with command #{command}"
      output = Puppet::Util::Execution.execute(full_command, options)
    end

    Puppet.info "listener result: #{output}"
  end

  def listener_processes
    return @listeners if @listeners

    kernel = Facter.value(:kernel)
    @listeners = []
    if kernel == 'windows'
      listener_services = `for /f "tokens=3" %i in ('tasklist /svc /fi "imagename eq tnslsnr.exe" ^| findstr "tnslsnr.exe"') do @echo %i`.split("\n")

      if Facter.value(:ora_install_homes)
        ora_install_homes_fact = Facter.value(:ora_install_homes)
        listener_services.map do |svc|
          regkey = Win32::Registry::HKEY_LOCAL_MACHINE.open("SYSTEM\\CurrentControlSet\\Services\\#{svc}")
          oracle_home = regkey['ImagePath'].gsub("\\BIN\\TNSLSNR", '')
          find_oracle_home = ora_install_homes_fact['product_version'].find { |oh| oh[0] =~ /#{Regexp.escape(oracle_home)}/i }
          oracle_home_name = find_oracle_home.nil? ? nil : find_oracle_home[1]['name']
          svc_home_name, lsnr_name = svc.scan(/Oracle(.*)TNSListener(.*)/).flatten
          service_state = `sc query #{svc}`.scan(/STATE *:.*(RUNNING)/).flatten.first
          Puppet.debug "listener_status #{service_state}"
          next unless service_state == 'RUNNING'
          next unless svc_home_name == oracle_home_name

          listener_name = lsnr_name.empty? ? 'LISTENER' : lsnr_name
          @listeners << { :os_user => resource[:os_user], :oracle_home_dir => oracle_home, :name => listener_name }
        end
      end
    else
      ps_bin = kernel != 'SunOS' || (kernel == 'SunOS' && Facter.value(:kernelrelease) == '5.11') ? '/bin/ps' : '/usr/ucb/ps'
      ps_arg = kernel == 'SunOS' ? 'awwx' : '-ef'

      command = "#{ps_bin} #{ps_arg} | /bin/grep -v grep | /bin/grep /bin/tnslsnr"

      Puppet.debug "listener_status #{command}"
      output = `#{command}`
      Puppet.debug output
      list = output.scan(/^(\S*?)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)/)
      list.each do |entry|
        @listeners << { :os_user => entry[0], :oracle_home_dir => entry[1].gsub('/bin/tnslsnr', ''), :name => entry[2] }
      end
    end
    @listeners
  end

  def start
    listener_control :start
  end

  def stop
    listener_control :stop
  end

  def restart
    listener_control :stop
    listener_control :start
  end

  def os_user
    current_listener && current_listener[:os_user]
  end

  def oracle_home_dir
    current_listener && current_listener[:oracle_home_dir]
  end

  def current_listener
    listener_processes.find { |e| e[:name].casecmp(resource[:listener_name]).zero? }
  end

  def ensure
    listener_processes.any? { |e| e[:name].casecmp(resource[:listener_name]).zero? } ? :start : :stop
  end

  def status
    # name = resource[:instance_name]

    kernel = Facter.value(:kernel)

    ps_bin = kernel != 'SunOS' || kernel == 'SunOS' && Facter.value(:kernelrelease) == '5.11' ? '/bin/ps' : '/usr/ucb/ps'
    ps_arg = kernel == 'SunOS' ? 'awwx' : '-ef'

    oracle_home = resource[:oracle_home_dir]
    listenername = resource[:listener_name]

    if Puppet::Util::Platform.windows?
      # TODO Add ORACLE_HOME_NAME in the findstr: findstr /i \"tnslsnr.exe.*Oracle.#{ORACLE_HOME_NAME}TNSListener\""
      command = "tasklist /svc | findstr /i \"tnslsnr.exe.*Oracle.*TNSListener\""
    else
      command = "#{ps_bin} #{ps_arg} | /bin/grep -v grep | /bin/grep -i #{oracle_home}/bin/tnslsnr.*#{listenername}"
    end

    Puppet.debug "listener_status #{command}"
    output = Puppet::Util::Execution.execute(command, { :failonfail => false, :combine => true })
    if Puppet::Util::Platform.windows?
      # TODO same as above
      output.scan(/tnslsnr.exe.*Oracle.*TNSListener/).empty? ? :stop : :start
    else
      output.scan(/#{oracle_home}\/bin\/tnslsnr.*#{listenername}/i).empty? ? :stop : :start
    end
  end
end
