#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ora_install/core'

Puppet::Type.newtype(:db_directory_structure) do
  def self.module_name; 'ora_install'; end
  desc 'add all the directories needed by the oracle db or asm installation'

  newparam(:name) do
    desc <<-DESCRIPTION
      The title.
    DESCRIPTION
    isnamevar
  end

  newproperty(:ensure) do
    desc 'Whether to do something.'

    newvalue(:present) do
      provider.configure
    end

    def retrieve
      create_orainst  = resource[:create_orainst_only]
      ora_inventory   = resource[:ora_inventory_dir]

      if create_orainst
        File.exist?(ora_inst_file) ? :present : :absent
      else
        oracle_base     = resource[:oracle_base_dir]
        download_folder = resource[:download_dir]

        File.exist?(oracle_base) && File.exist?(ora_inventory) && File.exist?(download_folder) ? :present : :absent
      end
    end

    def ora_inst_file
      case Facter.value(:kernel)
      when 'Linux' then '/etc/oraInst.loc'
      when 'SunOS' then '/var/opt/oracle/oraInst.loc'
      when 'windows' then 'C:\\Program Files\\Oracle\\Inventory\\oraInst.loc'
      else fail 'unsupported OS'
      end
    end
  end

  newparam(:oracle_base_dir) do
    desc <<-DESCRIPTION
      The oracle base folder.
    DESCRIPTION
    validate do |value|
      if value.nil?
        fail ArgumentError, 'oracle_base_dir cannot be empty'
      end
    end
  end

  newparam(:ora_inventory_dir) do
    desc <<-DESCRIPTION
      The oracle inventory folder.
    DESCRIPTION
    validate do |value|
      if value.nil?
        fail ArgumentError, 'ora_inventory_dir cannot be empty'
      end
    end
  end

  newparam(:download_dir) do
    desc <<-DESCRIPTION
      The download folder.
    DESCRIPTION
    validate do |value|
      if value.nil?
        fail ArgumentError, 'download_dir cannot be empty'
      end
    end
  end

  newparam(:os_user) do
    desc <<-DESCRIPTION
      The operating system user.
    DESCRIPTION

    defaultto 'oracle'
  end

  newparam(:os_group) do
    desc <<-DESCRIPTION
      The operating system group.
    DESCRIPTION

    defaultto 'oinstall'
  end

  newparam(:create_orainst_only) do
    desc <<-DESCRIPTION
      Only create oraInst.loc file.
    DESCRIPTION
    newvalues(true, false)
    defaultto false
  end
end
