require 'puppet_x/enterprisemodules/ora_install/core'

module PuppetX
  module EnterpriseModules
    # Docs
    module OraInstall
      # Docs
      module Tasks

        def self.extended(parent)
          parent.extend GetHomes
        end

        def log_msg(msg)
          @message << msg
        end

        def output(returncode, message)
          endtime = Time.now.iso8601
          json = {
            :return     => returncode,
            :ok         => true,
            :message    => message,
            :start_time => @starttime,
            :end_time   => endtime,
          }
          # TODO Make it pretty for task too, this will only make it pretty for plans
          if @format == 'pretty'
            puts JSON.pretty_generate(json)
          else
            puts message
          end
          exit returncode.to_i
        end

        def error_output(exitcode, message, kind)
          rc = exitcode.to_s.split.last
          endtime = Time.now.iso8601
          json = {
            :message => message,
            :ok      => false,
            :_error  => {
              :msg        => "Task exited : #{rc}\n#{message}",
              :kind       => kind,
              :details    => {
                :exitcode   => rc,
                :start_time => @starttime,
                :end_time   => endtime,
              },
            },
          }
          puts JSON.pretty_generate(json)
          exit rc.to_i
        end

        def startup(sid = nil, options = nil)
          database_action(:start, sid, options)
        end

        def shutdown(sid = nil, options = nil)
          database_action(:stop, sid, options)
        end

        def database_action(action, sid, options)
          Puppet::Util::Log.level = @verbose
          home    = oracle_home_for(sid)
          os_user = user_for_file(home)
          opts = {
            :name                    => 'startup',
            :instance_name           => sid,
            :os_user                 => os_user,
            :oracle_product_home_dir => home
          }

          provider = Puppet::Type.type(:db_control).new(opts).provider
          if provider.status == action
            return "Instance #{sid} already #{action}'ed."
          else
            provider.instance_control(action, options)
          end
        end

        def start_listener(sid = nil, listener_name)
          listener_action(:start, sid, listener_name)
        end

        def stop_listener(sid = nil, listener_name)
          listener_action(:stop, sid, listener_name)
        end

        def listener_action(action, sid, listener_name)
          Puppet::Util::Log.level = @verbose
          home    = oracle_home_for(sid)
          os_user = user_for_file(home)
          opts    = {
            :name            => 'start',
            :os_user         => os_user,
            :oracle_home_dir => home,
            :listener_name   => listener_name
          }

          provider = Puppet::Type.type(:db_listener).new(opts).provider
          if provider.status == action
            return "Listener \"#{listener_name}\" already #{action}'ed."
          else
            provider.listener_control(action)
          end

        end

        def copy(files_array, current_path, new_path)
          files_array.each do | file |
            current_file = "#{current_path}/#{file}"
            new_file     = "#{new_path}/#{file}"
            if File.exist?(current_file)
              uid = File.stat(current_file).uid
              owner = Etc.getpwuid(uid).name
              gid = File.stat(current_file).gid
              group = Etc.getgrgid(gid).name
              mode = File.stat(current_file).mode
              FileUtils.cp(current_file , new_file)
              FileUtils.chown(owner, group, new_file)
              FileUtils.chmod(mode, new_file)
              log_msg "File #{current_file} copied to #{new_file}, with owner #{owner}:#{group} and mode #{mode.to_s(8)[2..5]}."
            else
              log_msg "File #{current_file} not found. Skipping it."
            end
          end
        end
      end
    end
  end
end
