#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'
require 'yaml'
require 'fileutils'
require 'digest'
require 'time'

begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end

class Agent_action

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    @starttime = Time.now.iso8601
    fetch_input()
    execute()
  end

  def fetch_input
    params = JSON.parse(STDIN.read)
    #
    # Use loadpath specified in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_install/lib"
    require 'easy_type'
    require 'puppet_x/enterprisemodules/ora_install/tasks'
    extend ::PuppetX::EnterpriseModules::OraInstall::Tasks

    #
    # Fetch verbose level
    #
    @verbose = params.fetch('verbose') { 'info' }
    Puppet::Util::Log.level = @verbose

    @action       = params.fetch('action')
    @real_action  = @action == 'run' ? '-t' : "--#{@action}"
    comment       = params.fetch('comment') { '' }
    @real_comment = @real_action == '--disable' ? "'#{comment}'" : ''
    if @real_comment.empty? and @real_action == '--disable'
      fail "Please specify a comment when disabling puppet agent."
    end
    @format = params.fetch('format') { 'pretty' }
  end

  def execute
    begin
      options = { :failonfail => true, :combine => true }
      command = "/opt/puppetlabs/bin/puppet agent #{@real_action} #{@real_comment}"
      Puppet.debug command
      result = Puppet::Util::Execution.execute(command, options)
      Puppet.debug result
      output(0, "Puppet agent successfully #{@action}d")
    rescue Puppet::Error => e
      error_output(1, e.message, "enterprisemodules-ora_install/task-error")
    end
  end
end

Agent_action.run if __FILE__ == $0
